@@ -12,7 +12,8 @@ def processor(request):
         'PYBB_BUTTONS',
         'PYBB_DEFAULT_AVATAR_URL',
         'PYBB_MARKUP',
-        'PYBB_DEFAULT_TITLE'
+        'PYBB_DEFAULT_TITLE',
+        'PYBB_ENABLE_ANONYMOUS_POST'
         ):
         context[i] = getattr(defaults, i, None)
     context['PYBB_AVATAR_DIMENSIONS'] = '%sx%s' % (defaults.PYBB_AVATAR_WIDTH, defaults.PYBB_AVATAR_WIDTH)
