@@ -88,7 +88,13 @@ class ForumView(generic.ListView):
         self.forum = get_object_or_404(filter_hidden(self.request, Forum), pk=self.kwargs['pk'])
         if self.forum.category.hidden and (not self.request.user.is_staff):
             raise Http404()
-        return self.forum.topics.order_by('-sticky', '-updated').select_related()
+        qs = self.forum.topics.order_by('-sticky', '-updated').select_related()
+        if not (self.request.user.is_superuser or self.request.user in self.forum.moderators.all()):
+            if self.request.user.is_authenticated():
+                qs = qs.filter(Q(user=self.request.user)|Q(on_moderation=False))
+            else:
+                qs = qs.filter(on_moderation=False)
+        return qs
 
 
 class TopicView(generic.ListView):
@@ -99,11 +105,21 @@ class TopicView(generic.ListView):
 
     def get_queryset(self):
         self.topic = get_object_or_404(Topic.objects.select_related('forum'), pk=self.kwargs['pk'])
+        if self.topic.on_moderation and\
+           not pybb_topic_moderated_by(self.topic, self.request.user) and\
+           not self.request.user == self.topic.user:
+            raise PermissionDenied
         if (self.topic.forum.hidden or self.topic.forum.category.hidden) and (not self.request.user.is_staff):
             raise Http404()
         self.topic.views += 1
         self.topic.save()
-        return self.topic.posts.all().select_related('user')
+        qs = self.topic.posts.all().select_related('user')
+        if not pybb_topic_moderated_by(self.topic, self.request.user):
+            if self.request.user.is_authenticated():
+                qs = qs.filter(Q(user=self.request.user)|Q(on_moderation=False))
+            else:
+                qs = qs.filter(on_moderation=False)
+        return qs
 
     def get_context_data(self, **kwargs):
         ctx = super(TopicView, self).get_context_data(**kwargs)
@@ -115,6 +131,8 @@ class TopicView(generic.ListView):
             else:
                 ctx['form'] = PostForm(topic=self.topic)
             self.mark_read(self.request, self.topic)
+        elif defaults.PYBB_ENABLE_ANONYMOUS_POST:
+            ctx['form'] = PostForm(topic=self.topic)
         else:
             ctx['form'] = None
         if defaults.PYBB_FREEZE_FIRST_POST:
@@ -173,17 +191,17 @@ class AddPostView(FormChoiceMixin, generic.CreateView):
         forum = None
         topic = None
         if 'forum_id' in self.kwargs:
-            if not self.request.user.has_perm('pybb.add_topic'):
+            if not self.user.has_perm('pybb.add_topic'):
                 raise PermissionDenied
             forum = get_object_or_404(filter_hidden(self.request, Forum), pk=self.kwargs['forum_id'])
         elif 'topic_id' in self.kwargs:
             topic = get_object_or_404(Topic, pk=self.kwargs['topic_id'])
-            if topic.forum.hidden and (not self.request.user.is_staff):
+            if topic.forum.hidden and (not self.user.is_staff):
                 raise Http404
             if topic.closed:
                 raise PermissionDenied
         form_kwargs = super(AddPostView, self).get_form_kwargs()
-        form_kwargs.update(dict(topic=topic, forum=forum, user=self.request.user,
+        form_kwargs.update(dict(topic=topic, forum=forum, user=self.user,
                        ip=ip, initial={}))
         if 'quote_id' in self.request.GET:
             try:
@@ -194,8 +212,8 @@ class AddPostView(FormChoiceMixin, generic.CreateView):
                 post = get_object_or_404(Post, pk=quote_id)
                 quote = defaults.PYBB_QUOTE_ENGINES[defaults.PYBB_MARKUP](post.body, post.user.username)
                 form_kwargs['initial']['body'] = quote
-        if self.request.user.is_staff:
-            form_kwargs['initial']['login'] = self.request.user.username
+        if self.user.is_staff:
+            form_kwargs['initial']['login'] = self.user.username
         self.forum = forum
         self.topic = topic
         return form_kwargs
@@ -206,10 +224,23 @@ class AddPostView(FormChoiceMixin, generic.CreateView):
         ctx['topic'] = self.topic
         return ctx
 
-    @method_decorator(login_required)
+    def get_success_url(self):
+        if (not self.request.user.is_authenticated()) and defaults.PYBB_PREMODERATION:
+            return reverse('pybb:index')
+        return super(AddPostView, self).get_success_url()
+
     @method_decorator(csrf_protect)
-    @method_decorator(permission_required('pybb.add_post'))
     def dispatch(self, request, *args, **kwargs):
+        if request.user.is_authenticated():
+            self.user = request.user
+        else:
+            if defaults.PYBB_ENABLE_ANONYMOUS_POST:
+                self.user, new = User.objects.get_or_create(username=defaults.PYBB_ANONYMOUS_USERNAME)
+            else:
+                from django.contrib.auth.views import redirect_to_login
+                return redirect_to_login(request.get_full_path())
+        if not self.user.has_perm('pybb.add_post'):
+            raise PermissionDenied
         return super(AddPostView, self).dispatch(request, *args, **kwargs)
 
 class UserView(generic.DetailView):
@@ -231,10 +262,23 @@ class UserView(generic.DetailView):
 class PostView(generic.RedirectView):
     def get_redirect_url(self, **kwargs):
         post = get_object_or_404(Post, pk=self.kwargs['pk'])
+        if defaults.PYBB_PREMODERATION and\
+            post.on_moderation and\
+            (not pybb_topic_moderated_by(post.topic, self.request.user)) and\
+            (not post.user==self.request.user):
+            raise PermissionDenied
         count = post.topic.posts.filter(created__lt=post.created).count() + 1
         page = math.ceil(count / float(defaults.PYBB_TOPIC_PAGE_SIZE))
         return '%s?page=%d#post-%d' % (reverse('pybb:topic', args=[post.topic.id]), page, post.id)
 
+class ModeratePost(generic.RedirectView):
+    def get_redirect_url(self, **kwargs):
+        post = get_object_or_404(Post, pk=self.kwargs['pk'])
+        if not pybb_topic_moderated_by(post.topic, self.request.user):
+            raise PermissionDenied
+        post.on_moderation = False
+        post.save()
+        return post.get_absolute_url()
 
 class ProfileEditView(generic.UpdateView):
 
